require 'config_fun'
require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe "config_fun" do
  context "dsl" do
    it "should be able to use reserved words with an underscore" do
      struct = ConfigFun.parse! do
        _id 1
        _def 'dingle'
        _class 'bargle'
      end
      struct.should == {:class=>"bargle", :def=>"dingle", :id=>1}
    end

    it "should be able to nest hashes within hashes" do
      struct = ConfigFun.parse! do
        @person = "bob"
        entries do
          person do
            name @person
            height "5 feet"
          end
          shirt "green"
        end
      end
      struct.should == {:entries=>{:person=>{:height=>"5 feet", :name=>"bob"}, :shirt=>"green"}}
    end

    it "should convert multiple entries of the same hash key to an array" do
      struct = ConfigFun.parse! do
        entries do
          dingle do
            harble "barble"
          end
          people do
            person do
              name "john"
              height "5 feet"
            end
            person do
              name "bob"
              height "6 feet"
            end
          end
          shirt "green"
        end
      end
      struct.should == {:entries => {:dingle => {:harble => "barble"}, :people => {:person => [{:height=>"5 feet", :name=>"john" },{:height=>"6 feet", :name=>"bob"}]}, :shirt => 'green'}}
    end

    it "should be able to handle 3 or more array entries" do
      struct = ConfigFun.parse! do
        @people = ["sue","bob","sally","jim"]
        @people.each do |name|
          person do
            name name
          end
        end
      end
      struct.should == {:person => [{:name => "sue"}, {:name=>"bob"}, {:name=>"sally"}, {:name=>"jim"}]}
    end

    it "should see variables defined within its scope inside blocks" do
      struct = ConfigFun.parse! do
        @person = "bob"
        entries do
          person do
            brain do
              iq 100
            end
            name @person
            height "5 feet"
          end
          shirt "green"
        end
      end
      struct.should == {:entries=>{:person=>{:height=>"5 feet", :name=>"bob", :brain=>{:iq=>100}}, :shirt=>"green"}}
    end

    it "should be able to handle arrays both outer and inner" do
      struct = ConfigFun.parse! do
        entries do
          person do
            name "bob"
            colors do
              red "red"
              green "green"
            end
            colors do
              brown "brown"
              orange "orange"
            end
          end

          person do
            name "jim"
            colors do
              blue "blue"
              pink "pink"
            end
            colors do
              purple "purple"
              dark_brown "dark brown"
            end
          end
        end
      end
      struct.should == {:entries=>{:person=>[{:colors=>[{:red=>"red", :green=>"green"}, {:orange=>"orange", :brown=>"brown"}], :name=>"bob"}, {:colors=>[{:blue=>"blue", :pink=>"pink"}, {:purple=>"purple", :dark_brown=>"dark brown"}], :name=>"jim"}]}}
    end

    it "the base entry should be able to carry an array" do
      struct = ConfigFun.parse! do
        entry do
          item "item1"
        end

        entry do
          item "item2"
        end

        entry do
          item "item3"
        end
      end
      struct.should == {:entry=>[{:item=>"item1"}, {:item=>"item2"}, {:item=>"item3"}]}
    end
  end

  context "real-world example use" do
    let(:config) do
      config = ConfigFun.parse!(File.read(File.expand_path(File.dirname(__FILE__) + '/examples/test.rb')))
    end

    it "should parse a config correctly" do
      expected = {:config=>{:user=>[{:name=>"bob"}, {:name=>"jim"}]}}
      config.should == expected
    end
  end

  it "should warn the user if they pass arguments to a block" do
    $stderr.should_receive("<<").with(/Warning, ignoring args passed to a block/)
    struct = ConfigFun.parse! do
      user "bob" do
        room '123'
      end
    end

  end
end
